from lsr_benchmark.datasets import all_embeddings, all_datasets, IR_DATASET_TO_TIRA_DATASET, TIRA_DATASET_ID_TO_IR_DATASET_ID
from pathlib import Path
import os

def retrieve_command():
    import click
    class ClickParamTypeLsrDataset(click.ParamType):
        name = "dataset_or_dir"

        def convert(self, value, param, ctx):
            available_datasets = all_datasets()
            if value in available_datasets:
                return value
            
            if value in IR_DATASET_TO_TIRA_DATASET:
                return IR_DATASET_TO_TIRA_DATASET[value]

            if os.path.isdir(value):
                return os.path.abspath(value)

            msg = f"{value!r} is not a supported dataset " + \
            f"({', '.join(TIRA_DATASET_ID_TO_IR_DATASET_ID.get(i, i) for i in available_datasets)}) " + \
            "or a valid directory path"

            self.fail(msg, param, ctx)

    class ClickParamTypeLsrEmbedding(click.ParamType):
        name = "embedding_or_dir"

        def convert(self, value, param, ctx):
            if value:
                value = value.replace("/", "-")

            available_embeddings = all_embeddings()
            if value in available_embeddings:
                return "lightning-ir/" + value

            if os.path.isdir(value):
                return os.path.abspath(value)

            msg = f"{value!r} is not a supported embedding " + \
            f"({', '.join(available_embeddings)}) " + \
            "or a valid directory path"

            self.fail(msg, param, ctx)

    """A decorator that wraps a Click command with standard retrieval options."""
    def decorator(func):
        func = click.option(
            "--dataset",
            type=ClickParamTypeLsrDataset(),
            required=True,
            help="The dataset id or a local directory."
        )(func)

        func = click.option(
            "--output",
            required=True,
            type=Path,
            help="The directory where the output should be stored."
        )(func)

        func = click.option(
            "--embedding",
            type=ClickParamTypeLsrEmbedding(),
            required=False,
            default="naver/splade-v3",
            help="The embedding model."
        )(func)

        func = click.option(
            "--k",
            type=int,
            required=False,
            default=10,
            help="Number of results to return per each query."
        )(func)

        # Wrap as a command
        func = click.command()(func)
        return func
    return decorator



from operator import ge

from abc import ABC, abstractmethod
from tqdm import tqdm

from typing import Dict, List


class AbstractPassageChunker(ABC):
    @abstractmethod
    def process_batch(self, document_batch) -> List[Dict]:
        pass

    @staticmethod
    def chunk_document(document_sentences, sentences_word_count, passage_size) -> List[Dict]:
        """
        Creates the passage chunks for a given document
        """
        passages = []

        current_passage = ''
        current_passage_word_count = 0
        sub_id = 1

        for sentence, word_count in zip(document_sentences, sentences_word_count):
            if word_count >= passage_size:
                if current_passage:
                    passages.extend([{
                        "body": current_passage,
                        "id": sub_id
                    }, {
                        "body": sentence.text,
                        "id": sub_id+1
                    }])

                    sub_id += 2

                else:
                    passages.append({
                        "body": sentence.text,
                        "id": sub_id
                    })

                    sub_id += 1

            elif word_count + current_passage_word_count > passage_size:
                passages.append({
                    "body": current_passage,
                    "id": sub_id
                })

                current_passage = sentence.text
                current_passage_word_count = word_count
                sub_id += 1

            else:
                current_passage += ' ' + sentence.text + ' '
                current_passage_word_count += word_count

        if current_passage:
            passages.append({
                "body": current_passage,
                "id": sub_id
            })

        return passages

class SpacyPassageChunker(AbstractPassageChunker):
    def process_batch(self, document_batch, passage_size) -> None:
        import spacy
        nlp = spacy.load("en_core_web_sm", exclude=[
                         "parser", "tagger", "ner", "attribute_ruler", "lemmatizer", "tok2vec"])
        nlp.enable_pipe("senter")
        #python -m spacy download en_core_web_sm
        nlp.max_length = 2000000  # for documents that are longer than the spacy character limit
        doc_ids = list(document_batch.keys())
        ret = {}

        batch_document_texts = [document_batch[k] for k in doc_ids]
        processed_document_texts = nlp.pipe(batch_document_texts, n_process=1)

        for index, document in tqdm(enumerate(processed_document_texts), total=len(doc_ids)):
            document_sentences = list(document.sents)
            sentences_word_count = [
                len([token for token in sentence])
                for sentence in document_sentences
            ]

            generated_passages = self.chunk_document(document_sentences, sentences_word_count, passage_size)
            ret[doc_ids[index]] = generated_passages

        return ret

# code from https://github.com/grill-lab/trec-cast-tools
def segmented_document(documents, passage_size):
    print(f"Segment into passages of size {passage_size}.")
    chunker = SpacyPassageChunker()
    ret_passages = chunker.process_batch(documents, passage_size)
    ret = {}

    for k, p in ret_passages.items():
        segments = []
        if len(p) == 1:
            segments.append({"start": p[0]["id"], "end": p[0]["id"], "text": p[0]["body"].strip()})
        else:
            for i in range(len(p) - 1):
                start = p[i]
                end = p[i+1]

                segments.append({"start": start["id"], "end": end["id"], "text": (start["body"] + " " + end["body"]).strip()})

        ret[k] = {"doc_id": k, "segments": segments, "default_text": documents[k]}

    return ret



from tqdm import tqdm
import ir_datasets
from .corpus_subsampling import create_subsample
from .segmentation import segmented_document
import gzip
import json


def load_docs(ir_datasets_id, subsample):
    ret = {}
    docs_store = ir_datasets.load(ir_datasets_id).docs_store()

    skipped = 0
    for doc in tqdm(subsample):
        try:
            ret[doc] = docs_store.get(doc).default_text()[:2000000]
        except:
            skipped += 1
    print(f"Skipped {skipped} docs")
    return ret

def irds_id_from_config(config):
    ir_datasets_id = config["ir-datasets-id"]
    if ir_datasets_id.startswith("clueweb"):
        from ir_datasets_subsample import register_subsamples
        register_subsamples()
        ir_datasets_id = "corpus-subsamples/" + ir_datasets_id
    return ir_datasets_id

def materialize_raw_corpus(directory, subsample, config):
    ir_datasets_id = irds_id_from_config(config)
    docs = load_docs(ir_datasets_id, subsample)
    with gzip.open(directory / "corpus.jsonl.gz", 'wt') as f:
        for doc in docs.values():
            f.write(json.dumps(doc) + '\n')

def materialize_corpus(directory, config):
    ir_datasets_id = irds_id_from_config(config)
    if (directory/"corpus.jsonl.gz").is_file():
        return

    subsample = create_subsample(config["runs"], ir_datasets_id, config["subsample_depth"], directory)
    docs = load_docs(ir_datasets_id, subsample)
    docs = segmented_document(docs, config.get("passage_size", 200))
    with gzip.open(directory/"corpus.jsonl.gz", 'wt') as f:
        for doc in docs.values():
            f.write(json.dumps(doc) + '\n')

def materialize_queries(directory, config):
    from tira.ir_datasets_loader import IrDatasetsLoader
    
    irds_loader = IrDatasetsLoader()
    ir_datasets_id = irds_id_from_config(config)
    output_jsonl = directory / "queries.jsonl"
    output_xml = directory / "queries.xml"

    allowed_queries = set()
    for i in ir_datasets.load(ir_datasets_id).qrels_iter():
        allowed_queries.add(i.query_id)

    if not output_jsonl.exists():
        dataset = ir_datasets.load(ir_datasets_id)
        queries_mapped_jsonl = [irds_loader.map_query_as_jsonl(query, True) for query in dataset.queries_iter() if query.query_id in allowed_queries]
        with open(output_jsonl, 'w') as f:
            for l in queries_mapped_jsonl:
                f.write(l + '\n')

    if not output_xml.exists():
        dataset = ir_datasets.load(ir_datasets_id)
        queries_mapped_xml = [irds_loader.map_query_as_xml(query, True) for query in dataset.queries_iter() if query.query_id in allowed_queries]
        with open(output_xml, 'w') as f:
            for l in queries_mapped_xml:
                f.write(str(l) + '\n')

def materialize_qrels(output_qrels, config):
    ir_datasets_id = irds_id_from_config(config)

    if output_qrels.exists():
        return

    dataset = ir_datasets.load(ir_datasets_id)

    with open(output_qrels, 'w') as f:
        for qrel in dataset.qrels_iter():
            f.write(f"{qrel.query_id} 0 {qrel.doc_id} {qrel.relevance}\n")



import unittest
import ir_datasets
from lsr_benchmark import register_to_ir_datasets

class TestLoadEmbeddings(unittest.TestCase):
    def test_load_fails_for_non_existing_dataset(self):
        with self.assertRaises(Exception):
            register_to_ir_datasets("lsr-benchmark")

    def test_load_webis_splade_on_dl_19(self):
        register_to_ir_datasets("msmarco-passage/trec-dl-2019/judged")
        irds = ir_datasets.load("lsr-benchmark/msmarco-passage/trec-dl-2019/judged")

        doc_embeddings = irds.doc_embeddings(model_name="lightning-ir/webis/splade")
        query_embeddings = irds.query_embeddings(model_name="lightning-ir/webis/splade")

        self.assertIsNotNone(doc_embeddings)
        self.assertIsNotNone(query_embeddings)

    def test_load_on_robust04(self):
        register_to_ir_datasets("disks45/nocr/trec-robust-2004/fold1")
        irds = ir_datasets.load("lsr-benchmark/disks45/nocr/trec-robust-2004/fold1")

        doc_embeddings = irds.doc_embeddings(model_name="lightning-ir/webis/splade")
        query_embeddings = irds.query_embeddings(model_name="lightning-ir/webis/splade")

        self.assertIsNotNone(doc_embeddings)
        self.assertIsNotNone(query_embeddings)


import unittest
import json
from pathlib import Path
from approvaltests import verify_as_json
from lsr_benchmark.corpus.segmentation import segmented_document

def load_docs():
    ret = {}
    for i in ["1", "2"]:
        ret[i] = json.loads((Path(__file__).parent / "resources" / f"example-dl-0{i}.json").read_text())["text"]
    return ret

class TestPassageChunking(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        import spacy.cli
        spacy.cli.download("en_core_web_sm")

    def test_load_docs(self):
        docs = load_docs()
        self.assertIsNotNone(docs)

    def test_chunking(self):
        docs = load_docs()
        actual = segmented_document(docs, 200)
        verify_as_json(actual)


import logging
import re
from collections import defaultdict
from glob import glob
from gzip import GzipFile
from io import TextIOWrapper
from pathlib import Path
from typing import TYPE_CHECKING, Mapping, Dict, Any
from zipfile import ZipFile
import gzip

import click
import ir_measures
import pandas as pd
import yaml
from tira.check_format import lines_if_valid
from ir_measures import parse_trec_measure

import lsr_benchmark
from lsr_benchmark.datasets import TIRA_DATASET_ID_TO_IR_DATASET_ID, all_embeddings

if TYPE_CHECKING:
    from typing import _KT, _T, _VT, Any, Callable, Literal, Optional, Union

    from ir_measures import Measure, ScoredDoc

    Metadata = dict[str, Any]


def __get_nested(
    d: "Mapping[_KT, Union[dict, _VT]]", keys: "list[_KT]"
) -> "Union[Mapping[_KT, Union[dict, _VT]], _VT]":
    """Recursively retrieves a value from a nested mapping using a list of keys.

    Args:
        d (Mapping[_KT, Union[dict, _VT]]): The dictionary to traverse.
        keys (list[_KT]): A list of keys representing the path to the desired value.

    Raises:
        TypeError: If an intermediate value in the path is not a mapping.
        KeyError: If any key in the path is not found in the corresponding mapping.

    Returns:
        The value located at the nested key path.
    """
    out: "Union[Mapping[_KT, Union[dict, _VT]], _VT]" = d
    for i, key in enumerate(keys):
        if not isinstance(out, Mapping):
            raise TypeError(f"The value at {'>'.join(map(str, keys[:i]))} is not a mapping. Have {out}.")
        if key not in out:
            raise KeyError(f"The key {'>'.join(map(str, keys[:i+1]))} could not be found. Have {out}.")
        out = out[key]
    return out


def __get_nested_or_default(d: "Mapping[_KT, Union[dict, _VT]]", keys: "list[_KT]", default: "_T" = None) -> "Union[_VT, _T]":
    try:
        return __get_nested(d, keys)
    except KeyError:
        return default



def __read_metrics(name: str) -> "tuple[dict[str, Metadata], list[ScoredDoc]]":
    metadata: "dict[str, Metadata]" = {}

    if name.endswith('/run.txt.gz'):
        name = name.replace('/run.txt.gz', '/')

    if Path(name).is_dir():
        for l in lines_if_valid(Path(name), "ir_metadata"):
            metadata[l['name'].replace('.', '').split("-")[0]] = l['content']
        if (Path(name) / "run.txt").is_file():
            run = list(ir_measures.read_trec_run((Path(name) / "run.txt").read_text()))
        else:
            run = list(ir_measures.read_trec_run(gzip.open(Path(name) / "run.txt.gz", "rt")))
    else:
        with ZipFile(name) as archive:
            for entry in archive.filelist:
                if (m := re.match(r"(\w+)-metadata.ya?ml", entry.filename)) is not None:
                    with archive.open(entry) as file:
                        metadata[m.group(1)] = yaml.safe_load(file)
            with archive.open("run.txt.gz", mode="r") as compressed:
                with GzipFile(fileobj=compressed, mode="r") as binary:
                    with TextIOWrapper(binary, encoding="utf-8") as file:
                        run = list(ir_measures.read_trec_run(file))
    
    if len(metadata) == 0:
        raise ValueError("I could not read any metadata")
    if len(run) == 0:
        raise ValueError("I could not load a run")
    return metadata, run


def __get_runtime(metadata: "Metadata", param: "Literal['system', 'user', 'wallclock']" = "wallclock") -> "Optional[str]":
    return __get_nested_or_default(metadata, ("resources", "runtime", param))


def __get_energy_usage(metadata: "Metadata", param: "Literal['total', 'cpu', 'gpu', 'ram']" = "total") -> "Optional[float]":
    def __get_energy(device: "str") -> float:
        try:
            energy_str = metadata["resources"][device]["energy used system"]
        except KeyError:
            logging.warning(f"Energy for {device} was not reported; using 0 Joules")
        try:
            if match := re.match(r'^(\S+)\s*J$', energy_str):
                return float(match[1])
            else:
                raise ValueError
        except ValueError:
            logging.error(f"Could not parse energy string: '{energy_str}'")
        return 0
    if param == "total":
        return __get_energy("cpu") + __get_energy("gpu") + __get_energy("ram")
    return __get_energy(param)


def __get_avg_cpu_usage(metadata: "Metadata") -> "Optional[int]":
    return __get_nested_or_default(metadata, ("resources", "cpu", "used process", "avg"))


def __get_max_ram_usage(metadata: "Metadata") -> "Optional[int]":
    return __get_nested_or_default(metadata, ("resources", "ram", "used process", "max"))


def __get_max_vram_usage(metadata: "Metadata") -> "Optional[int]":
    return __get_nested_or_default(metadata, ("resources", "gpu", "used vram process", "max"))


def __get_avg_gpu_usage(metadata: "Metadata") -> "Optional[int]":
    return __get_nested_or_default(metadata, ("resources", "gpu", "used process", "avg"))


__efficiency_measures: "dict[str, Callable]" = {
    "runtime": __get_runtime, "energy": __get_energy_usage, "cpu": __get_avg_cpu_usage, "ram": __get_max_ram_usage,
    "gpu": __get_avg_gpu_usage, "vram": __get_max_vram_usage
}


def __parse_tirex_measure(measure: "str") -> "Callable":
    name, *arg = measure.split('_', 2)
    func = __efficiency_measures[name]
    return lambda x: func(x, *arg)


def __parse_measure(measure: "str") -> "tuple[str, Literal['ir_measure', 'tirex'], Measure | Callable]":
    try:
        return (measure, 'ir_measure', parse_trec_measure(measure)[0])
    except ValueError:
        try:
            return (measure, 'ir_measure', ir_measures.parse_measure(measure))
        except:
            pass

    return (measure, 'tirex', __parse_tirex_measure(measure))


def __get_dataset_name(metadata: Dict[str, Any]) -> str:
    candidates = set()

    for k, m in metadata.items():
        if "data" in m and "test collection" in m["data"] and "name" in m["data"]["test collection"] and m["data"]["test collection"]["name"]:
            candidates.add(m["data"]["test collection"]["name"])

    candidates = [i for i in candidates if i != '/tira-data/input']
    if len(candidates) != 1:
        raise ValueError(f"I can not extract the dataset from the metadata. I found candidates: {list(candidates)}")

    return list(candidates)[0]


def __get_embedding_name(p: Path):
    # FIXME read this from metadata
    ret = []
    for embedding in all_embeddings():
        if embedding in str(Path(p)).split("/"):
            ret += [embedding]
    if '/none/' in str(p):
        return None
    if len(ret) != 1:
        #raise ValueError(f"can not process {p}")
        return None
    return ret[0]

def __get_output_routine(specifier: str) -> "Callable[[pd.DataFrame], None]":
    suffix_to_routine: dict[str, Callable[[pd.DataFrame], None]] = {
        ".csv": lambda df: df.to_csv(specifier),
        ".xlsx": lambda df: df.to_excel(specifier),
        ".htm": lambda df: df.to_html(specifier),
        ".html": lambda df: df.to_html(specifier),
        ".json": lambda df: df.to_json(specifier),
        ".gz": lambda df: df.to_json(specifier, lines=True, orient="records"),
        ".tex": lambda df: df.to_latex(specifier),
        ".md": lambda df: df.to_markdown(specifier),
        ".parquet": lambda df: df.to_parquet(specifier),
    }

    if specifier == "-":
        return lambda i: print(pd.DataFrame({j["approach"]: j.to_dict() for _, j in i.iterrows()}))
    elif (routine := suffix_to_routine.get(Path(specifier).suffix, None)) is not None:
        return routine
    else:
        raise ValueError(f"The suffix of {specifier} is not known.")


def evaluate_approach(approach: str, measure: list[str]):
    ret = {}
    metadata, run = __read_metrics(approach)
    for group, meta in metadata.items():
        for name, typ, func in measure:
            if typ == 'tirex':
                if (val := func(meta)) is not None:
                    ret[f"{group}.{name}"] = val
                else:
                    ret[f"{group}.{name}"] = None
                    logging.warning(
                        f"Measure {name} could not be reported for {approach}.{group} as its metadata is not present")
    # Update with ir_measures effectiveness measures
    irmeasures = set(m for _, t, m in measure if t == 'ir_measure')

    dataset = __get_dataset_name(metadata)
    lsr_benchmark.register_to_ir_datasets(dataset)
    dset = lsr_benchmark.load(dataset)
    assert dset.has_qrels()

    ret.update({str(k): v for k, v in ir_measures.calc_aggregate(irmeasures, dset.qrels, run).items()})
    ret["tira-dataset-id"] = dataset
    ret["ir-dataset-id"] = TIRA_DATASET_ID_TO_IR_DATASET_ID[dataset]
    ret["approach"] = approach
    ret["embedding/model"] = __get_embedding_name(approach)
    return ret


@click.argument(
    "approaches",
    type=str,
    nargs=-1,
)
@click.option(
    "-m", "--measure",
    type=__parse_measure,
    required=False,
    multiple=True,
    default=["ndcg_cut.10", "nDCG(judged_only=True)@10", "P_10", "RR", "runtime_wallclock", "energy_total"],
    help="The dataset id or a local directory.",
)
@click.option(
    "--upload",
    type=bool,
    default=False,
    is_flag=True,
    required=False,
    help="Upload to tira.",
)
@click.option(
    "-o", "--out",
    type=str,
    required=False,
    multiple=False,
    default="-",
    help="The output file to write to. Use - to print the results to stdout. Default: -",
)
def evaluate(approaches: list[str], measure: list[str], out: str, upload: bool) -> int:
    approaches = [x for xs in map(glob, approaches) for x in xs]
    output_routine = __get_output_routine(out)

    scores: list = []
    from tqdm import tqdm
    for approach in tqdm(approaches):
        scores += [evaluate_approach(approach, measure)]

        if upload:
            from tira.tira_cli import upload_command
            from lsr_benchmark.irds import TIRA_LSR_TASK_ID
            upload_command(dataset=dataset, directory=approach, dry_run=False, system=approach, default_task=TIRA_LSR_TASK_ID)

    output_routine(pd.DataFrame(scores))
    return 0



import unittest
from pathlib import Path

from lsr_benchmark import register_to_ir_datasets
from lsr_benchmark.datasets import IR_DATASET_TO_TIRA_DATASET
import ir_datasets

class TestIrdsIntegration(unittest.TestCase):
    def test_fails_for_non_existing_dataset(self):
        with self.assertRaises(Exception):
            register_to_ir_datasets("this-does-not-exist")

    def test_works_for_none_as_dataset(self):
        register_to_ir_datasets()

    def test_from_local_directory(self):
        resource_dir = str(Path(__file__).parent / "resources" / "example-dataset")
        register_to_ir_datasets(resource_dir)
        ds = ir_datasets.load(resource_dir)

        self.assertEqual(3, len(list(ds.queries_iter())))
        self.assertEqual(4, len(list(ds.docs_iter())))


    def test_from_local_directory_with_prefix(self):
        resource_dir = str(Path(__file__).parent / "resources" / "example-dataset")
        register_to_ir_datasets(resource_dir)
        ds = ir_datasets.load("lsr-benchmark/" + resource_dir)

        self.assertEqual(3, len(list(ds.queries_iter())))
        self.assertEqual(4, len(list(ds.docs_iter())))

    def test_ms_marco_dataset(self):
        register_to_ir_datasets("msmarco-passage/trec-dl-2019/judged")
        ds = ir_datasets.load("lsr-benchmark/msmarco-passage/trec-dl-2019/judged")

        self.assertEqual(43, len(list(ds.queries_iter())))
        self.assertEqual(32123, len(list(ds.docs_iter())))
        self.assertEqual(9260, len(list(ds.qrels_iter())))

    def test_all_datasets_can_be_loaded(self):
        for i in range(3):
            for ds in IR_DATASET_TO_TIRA_DATASET.keys():
                register_to_ir_datasets(ds)
                self.assertIsNotNone(ir_datasets.load(f"lsr-benchmark/{ds}"))

            for ds in IR_DATASET_TO_TIRA_DATASET.values():
                register_to_ir_datasets(ds)
                self.assertIsNotNone(ir_datasets.load(ds))



import json
from pathlib import Path

def lsr_overview():
    return json.loads((Path(__file__).parent / "overview.json").read_text())

def all_embeddings():
    overview = lsr_overview()
    ret = set()

    for dataset_id, stats in overview.items():
        for embedding, embedding_size in stats['embedding-sizes'].items():
            ret.add(embedding)
    return sorted(list(ret))

def all_datasets():
    overview = lsr_overview()
    return sorted(list(overview.keys()))


TIRA_DATASET_ID_TO_IR_DATASET_ID = {
    'trec-18-web-20251008-test': 'clueweb09/en/trec-web-2009',
    'trec-19-web-20251008-test': 'clueweb09/en/trec-web-2010',
    'trec-20-web-20251008-test': 'clueweb09/en/trec-web-2011',
    'trec-21-web-20251008-test': 'clueweb09/en/trec-web-2012',
    'trec-22-web-20251008-test': 'clueweb12/trec-web-2013',
    'trec-23-web-20251008-test': 'clueweb12/trec-web-2014',
    'trec-28-deep-learning-passages-20250926-training': 'msmarco-passage/trec-dl-2019/judged',
    'trec-28-misinfo-20251008_1-test': 'clueweb12/b13/trec-misinfo-2019',
    'trec-29-deep-learning-passages-20250926-training': 'msmarco-passage/trec-dl-2020/judged',
    'trec-33-rag-20250926_1-training': 'msmarco-segment-v2.1/trec-rag-2024',
    'trec-robust-2004-fold-1-20250927-test': 'disks45/nocr/trec-robust-2004/fold1',
    'trec-robust-2004-fold-2-20250926-test': 'disks45/nocr/trec-robust-2004/fold2',
    'trec-robust-2004-fold-3-20250926-test': 'disks45/nocr/trec-robust-2004/fold3',
    'trec-robust-2004-fold-4-20250926-test': 'disks45/nocr/trec-robust-2004/fold4',
    'trec-robust-2004-fold-5-20250926-test': 'disks45/nocr/trec-robust-2004/fold5'
}

IR_DATASET_TO_TIRA_DATASET = {v:k for k, v in TIRA_DATASET_ID_TO_IR_DATASET_ID.items()}

def all_ir_datasets():
    return sorted([TIRA_DATASET_ID_TO_IR_DATASET_ID[i] for i in all_datasets() if i in TIRA_DATASET_ID_TO_IR_DATASET_ID and TIRA_DATASET_ID_TO_IR_DATASET_ID[i]])



import click
from tira.rest_api_client import Client
from pathlib import Path
from lsr_benchmark.datasets import all_embeddings, all_ir_datasets, IR_DATASET_TO_TIRA_DATASET
from shutil import copytree

@click.option(
    "--dataset",
    type=click.Choice(all_ir_datasets()),
    required=True,
)
@click.option(
    "--embedding",
    type=click.Choice(all_embeddings()),
    required=True,
)
@click.option(
    "-o", "--out",
    type=str,
    required=False,
    multiple=False,
    default=None,
    help="The output directory to write to.",
)
def download_embeddings(dataset, embedding, out):
    tira = Client()
    ret = tira.get_run_output(f'lsr-benchmark/lightning-ir/{embedding}', IR_DATASET_TO_TIRA_DATASET[dataset])
    if out is not None:
        copytree(ret, out)
        ret = out
    print(ret)


@click.option(
    "--dataset",
    type=click.Choice(all_ir_datasets()),
    required=True,
)
@click.option(
    "--embedding",
    type=click.Choice(all_embeddings()),
    required=True,
)
@click.option(
    "--retrieval",
    # TODO Make this generic
    type=click.Choice(sorted(["seismic", "duckdb", "kannolo", "naive-search", "pyterrier-splade-pisa", "pyterrier-splade", "pytorch-naive", "seismic"])),
    required=True,
)
@click.option(
    "-o", "--out",
    type=str,
    required=False,
    multiple=False,
    default=None,
    help="The output directory to write to.",
)
def download_run(dataset, embedding, retrieval, out):
    tira = Client()
    ret = tira.get_run_output(f'lsr-benchmark/lightning-ir/{embedding}', IR_DATASET_TO_TIRA_DATASET[dataset])
    if out is not None:
        copytree(ret, out)
        ret = out
    print(ret)


import zipfile
from pathlib import Path
from typing import List, NamedTuple, TYPE_CHECKING

import numpy as np
from ir_datasets.datasets.base import Dataset
from ir_datasets.formats import BaseDocs, BaseQueries, GenericQuery, TrecQrels
from ir_datasets.util import MetadataComponent
from tira.check_format import JsonlFormat, QueryProcessorFormat
from tira.third_party_integrations import in_tira_sandbox
from tqdm import tqdm
import os
from glob import glob

if TYPE_CHECKING:
    from typing import Optional

MAPPING_OF_DATASET_IDS = {"clueweb09/en/trec-web-2009": "data/trec-18-web"}
TIRA_LSR_TASK_ID = "lsr-benchmark"


_IR_DATASETS_FROM_TIRA = None

def embeddings(
        dataset_id: str, model_name: str, text_type: str
    ) -> tuple[np.ndarray, np.ndarray, np.ndarray, list[str]]:

    if Path(model_name).is_dir() and (Path(model_name) / text_type).is_dir() and (Path(model_name) / text_type / f"{text_type}-embeddings.npz").exists():
        embedding_dir = Path(model_name) / text_type
    else:
        from tira.rest_api_client import Client
        tira = Client()

        team_and_model = model_name.split('/')
        team_name = team_and_model[0]
        model_name = '-'.join(team_and_model[1:])
        if in_tira_sandbox():
             embedding_dir = tira.input_run_in_sandbox(f"{TIRA_LSR_TASK_ID}/{team_name}/{model_name}")
             if not embedding_dir:
                 raise ValueError("not mounted")
             embedding_dir = Path(embedding_dir) / text_type
        else:
            embedding_dir = tira.get_run_output(f"{TIRA_LSR_TASK_ID}/{team_name}/{model_name}", dataset_id) / text_type

    embeddings = np.load(embedding_dir / f"{text_type}-embeddings.npz")

    try:
        from tirex_tracker import register_file
        for i in glob(f"{embedding_dir}/*.yml") + glob(f"{embedding_dir}/*.yaml"):
            register_file(embedding_dir, i.split("/")[-1], subdirectory=".embedding")
    except:
        pass

    ids = (embedding_dir / f"{text_type}-ids.txt").read_text().strip().split("\n")

    ret = []
    indices = embeddings["indices"].astype("U30")
    data = embeddings["data"]

    ptr_start = 0
    for doc_id, ptr_end in tqdm(list(zip(ids, embeddings["indptr"][1:])), "load doc embeddings"):
        tokens = indices[ptr_start:ptr_end]
        values = data[ptr_start:ptr_end]
        ptr_start = ptr_end
        ret.append((doc_id, tokens, values))
    return ret


def ir_datasets_from_tira(force_reload=False):
    global _IR_DATASETS_FROM_TIRA
    
    if in_tira_sandbox():
        return []
    
    if _IR_DATASETS_FROM_TIRA is None or force_reload:
        from tira.rest_api_client import Client
        tira = Client()
        _IR_DATASETS_FROM_TIRA = list(tira.datasets(TIRA_LSR_TASK_ID, force_reload).keys())

    return _IR_DATASETS_FROM_TIRA


def extracted_resource(irds_id: str, f) -> Path:
    if os.path.isdir(irds_id):
        return Path(irds_id)
    else:
        raise ValueError("ToDo: finish refactoring")


def _dowload_from_tira(ir_datasets_id, truth_dataset):
    if os.path.isdir(ir_datasets_id):
        return Path(ir_datasets_id)

    from tira.rest_api_client import Client
    tira = Client()
    return tira.download_dataset(task=None, dataset=ir_datasets_id, truth_dataset=truth_dataset)


class Segment(NamedTuple):
    offset_start: int
    offset_end: int
    text: str


class LsrBenchmarkDocument(NamedTuple):
    doc_id: str
    segments: List[Segment]

    @staticmethod
    def _from_json(json_doc):
        segments = [Segment(int(i["start"]), int(i["end"]), i["text"]) for i in json_doc["segments"]]
        return LsrBenchmarkDocument(json_doc["doc_id"], segments)

    def default_text(self):
        return "" if len(self.segments) == 0 else self.segments[0].text


class LsrBenchmarkDocumentEmbedding(NamedTuple):
    doc_id: str
    embedding: np.array


class LsrBenchmarkSegmentedDocument(NamedTuple):
    doc_id: str
    segment: Segment

    def default_text(self):
        return self.segment.text


class LsrBenchmarkQueries(BaseQueries):
    def __init__(self, ir_datasets_id):
        self.__irds_id = ir_datasets_id
        self.__queries_file = None

    def queries_iter(self):
        if not self.__queries_file:
            self.__queries_file = _dowload_from_tira(self.__irds_id, False) / "queries.jsonl"

        for l in QueryProcessorFormat().all_lines(self.__queries_file):
            yield GenericQuery(l["qid"], l["query"])


class LsrBenchmarkQueryEmbedding(NamedTuple):
    query_id: str
    embedding: np.array


class LsrBenchmarkDocuments(BaseDocs):
    def __init__(self, irds_id):
        self.__corpus_file = None
        self.__docs = None
        self.__irds_id = irds_id

    def docs_iter(self):
        for l in self.docs():
            yield LsrBenchmarkDocument._from_json(l)

    def docs(self):
        if not self.__docs:
            if not self.__corpus_file:
                self.__corpus_file = _dowload_from_tira(self.__irds_id, False) / "corpus.jsonl.gz"
            reader = JsonlFormat()
            reader.apply_configuration_and_throw_if_invalid(
                {"required_fields": ["doc_id", "segments"], "max_size_mb": 2500}
            )
            self.__docs = reader.all_lines(self.__corpus_file)
        return self.__docs

    def docs_count(self):
        return len([1 for i in self.docs_iter()])


class LsrBenchmarkSegmentedDocuments(LsrBenchmarkDocuments):
    def docs_iter(self, embedding=None):
        for doc in super().docs_iter(embedding):
            for idx, segment in zip(range(len(doc.segments)), doc.segments):
                yield LsrBenchmarkSegmentedDocument(f"{doc.doc_id}___{idx}___", segment)

    def docs_count(self):
        return len([1 for i in self.docs_iter()])


class LsrBenchmarkDataset(Dataset):
    def __init__(self, ir_datasets_id, segmented=False, documentation=None):
        self.__irds_id = ir_datasets_id

        queries = LsrBenchmarkQueries(ir_datasets_id)

        if in_tira_sandbox():
            qrels_obj = None
        else:
            class QrelsObj:
                def stream(self):
                    qrels_file = _dowload_from_tira(ir_datasets_id, True) / "qrels.txt"
                    return qrels_file.open("rb")

            qrels_obj = TrecQrels(QrelsObj(), {0: "Not Relevant", 1: "Relevant"})

        if segmented:
            docs = LsrBenchmarkSegmentedDocuments(ir_datasets_id)
        else:
            docs = LsrBenchmarkDocuments(ir_datasets_id)

        super().__init__(docs, queries, qrels_obj, documentation)
        self.metadata = MetadataComponent(ir_datasets_id, self)

    def query_embeddings(self, model_name: str) -> tuple[np.ndarray, np.ndarray, np.ndarray, list[str]]:
        return embeddings(self.__irds_id, model_name, "query")

    def doc_embeddings(self, model_name: str) -> tuple[np.ndarray, np.ndarray, np.ndarray, list[str]]:
        return embeddings(self.__irds_id, model_name, "doc")


def extract_zip(zip_file: Path, target_directory: Path):
    if target_directory.exists():
        return

    if not zipfile.is_zipfile(zip_file):
        raise ValueError(f"I expected that {zip_file} is not a valid ZIP archive.")

    with zipfile.ZipFile(zip_file, "r") as zip_ref:
        target_directory.mkdir(parents=True, exist_ok=True)
        zip_ref.extractall(target_directory)


def build_dataset(ir_datasets_id: str, segmented: bool):
    try:
        from tirex_tracker import register_metadata
        register_metadata({"data": {"test collection": {"name": ir_datasets_id}}})
    except:
        pass

    return LsrBenchmarkDataset(
        ir_datasets_id=ir_datasets_id,
        segmented=segmented,
    )



import click
import os
import sys
from tira.io_utils import _fmt, log_message, verify_tira_installation
from tira.third_party_integrations import temporary_directory
from tira.check_format import _fmt, check_format
from tira.rest_api_client import Client
from pathlib import Path
from lsr_benchmark.datasets import all_embeddings, all_datasets
import shutil
import yaml
import json


def run_foo(docker_image, command, dataset_id, embedding, output_dir=None):
    if output_dir is not None and Path(output_dir).exists():
        return
    tira = Client()
    dataset_path = tira.download_dataset("lsr-benchmark", dataset_id)
    if embedding.lower() != "none":
        embeddings_dir = tira.get_run_output(f'lsr-benchmark/lightning-ir/{embedding}', dataset_id)
    else:
        embeddings_dir = None
    tmp_dir = temporary_directory()
    tira.local_execution.run(
        image=docker_image,
        command=command,
        input_dir=dataset_path,
        output_dir=tmp_dir,
        allow_network=False,
        input_run=embeddings_dir
    )

    result, msg = check_format(Path(tmp_dir), ["run.txt"], {})
    if result != _fmt.OK:
        print(msg)
        raise ValueError(msg)

    tag = yaml.safe_load((Path(tmp_dir) / "retrieval-metadata.yml").read_text())["tag"]
    
    if output_dir is not None:
        from tira.io_utils import patch_ir_metadata
        output_dir.parent.mkdir(parents=True, exist_ok=True)
        shutil.copytree(tmp_dir, output_dir)
        patch_ir_metadata(output_dir, {"data": {"test collection": {"name": "/tira-data/input"}}}, {"data": {"test collection": {"name": dataset_id}}})
    
    return tag


@click.argument(
    "approaches",
    type=str,
    nargs=-1,
)
@click.option(
    "-o", "--out",
    type=str,
    required=True,
    multiple=False,
    help="The output directory to write to.",
)
@click.option(
    "--dataset",
    type=click.Choice(["all"] + all_datasets()),
    multiple=True,
    help="The datasets to run on.",
)
@click.option(
    "--embedding",
    type=click.Choice(["all", "none", ] + all_embeddings()),
    multiple=True,
    help="The datasets to run on.",
)
def retrieval(approaches: list[str], dataset: list[str], embedding: list[str], out: str) -> int:
    all_messages = []

    def print_message(message, level):
        all_messages.append((message, level))
        os.system("cls" if os.name == "nt" else "clear")
        print(' '.join([sys.argv[0].split('/')[-1]] + sys.argv[1:]))
        for m, l in all_messages:
            log_message(m, l)

    if dataset is None or not dataset or "all" in dataset:
        dataset = all_datasets()

    if embedding is None or not embedding or "all" in embedding:
        embedding = all_embeddings()
    if embedding and "none" in embedding:
        embedding = ["none"]

    status = verify_tira_installation()

    if status != _fmt.OK:
        print_message("Your TIRA installation is not valid. Please run 'tira-cli verify-installation' to resolve the problem", status)
        return 1

    print_message("Your TIRA installation is valid.", _fmt.OK)
    
    tira = Client()
    approach_to_execution = {}
    for approach in approaches:
        docker_tag, zipped_code, remotes, commit, active_branch = tira.build_docker_image_from_code(
            Path(approach), log_message, False
        )
        assert docker_tag not in approach_to_execution.values()
        cmd = (Path(approach) / "README.md").read_text().split("tira-cli code-submission")[1].split('--command')[1].split("'")[1]

        log_message(f"Approach {approach} is compiled.", _fmt.OK)
        system_tag = run_foo(docker_tag, cmd, 'tiny-example-20251002_0-training', embedding[0])
        print_message(f"Approach {approach} compiled and produced valid outputs on example dataset (tag={system_tag}).", _fmt.OK)
        approach_to_execution[approach] = {"tag": docker_tag, "command": cmd}

    stats = {}
    for d in dataset:
        for e in embedding:
            for approach in approaches:
                out_dir = Path(out) / d / e / approach
                try:
                    run_foo(approach_to_execution[approach]["tag"], approach_to_execution[approach]["command"], d, e, out_dir)
                except:
                    continue
                    log_message(f"Approach {approach} finished on {d} for embedding {e}", _fmt.OK)
                    if approach not in stats:
                        stats[approach] = {"datasets": set(), "embeddings": set()}
                    stats[approach]["datasets"].add(d)
                    stats[approach]["embeddings"].add(e)
    for approach in stats:
        print_message(f"Approach {approach} produced valid outputs on {len(stats[approach]['datasets'])} datasets for {len(stats[approach]['embeddings'])} embeddings.", _fmt.OK)
    
    return 0



# The code is taken from https://github.com/webis-de/ecir25-corpus-subsampling/blob/main/tests/test_run_pool_corpus_sampler.py
import gzip
import json
from abc import ABC, abstractmethod
from pathlib import Path
from tqdm import tqdm
from glob import glob

import ir_datasets


class CorpusSampler(ABC):
    """Sample a set of documents from a large corpus."""

    @abstractmethod
    def sample_corpus(ir_datasets_id: str, runs: list) -> set[str]:
        """Sample a corpus (returned as a set of document IDs)
        for a given dataset and a set of runs that were used
        to cunstruct the pool.

        Args:
            ir_datasets_id (str): The ir_datasets ID of the dataset.
            runs (list): The runs used to construct the pool.

        Returns:
            set[str]: The sampled corpus as a set of document IDs.
        """
        pass


class JudgmentPoolCorpusSampler(CorpusSampler):
    def sample_corpus(self, ir_datasets_id: str, runs: list) -> set[str]:
        """Sample a corpus (returned as a set of document IDs)
        by just returning all judged documents from the judgment pool.

        Args:
            ir_datasets_id (str): The ir_datasets ID of the dataset.
            runs (list): The runs used to construct the pool.

        Returns:
            set[str]: The judged documents as sampled corpus.
        """
        qrels_iter = ir_datasets.load(ir_datasets_id).qrels_iter()
        ret = set()

        for qrel in qrels_iter:
            ret.add(qrel.doc_id)

        return ret

    def __str__(self) -> str:
        return "judgment-pool"


class RunPoolCorpusSampler(JudgmentPoolCorpusSampler):
    def __init__(self, depth: int):
        """Create a pool of the passed depth for the passed runs as sampled corpus.

        Args:
            depth (int): The depth of the pool
        """
        self.depth = depth

    def sample_corpus(self, ir_datasets_id: str, runs: list) -> set[str]:
        """Sample a corpus (returned as a set of document IDs)
        by returning the top-k pool of all runs that formed
        the original judgment pool.

        Args:
            ir_datasets_id (str): The ir_datasets ID of the dataset.
            runs (list): The runs used to construct the pool.

        Returns:
            set[str]: The top-k pool of the runs as sampled corpus
        """
        from trectools import TrecPoolMaker
        ret = super().sample_corpus(ir_datasets_id, runs)
        qrels_iter = ir_datasets.load(ir_datasets_id).qrels_iter()
        allowed_query_ids = set()

        for qrel in qrels_iter:
            allowed_query_ids.add(str(qrel.query_id))

        pool = TrecPoolMaker().make_pool(runs, strategy="topX", topX=self.depth).pool
        skipped = 0
        for qid in pool.keys():
            docids = pool[qid]
            if str(qid) not in allowed_query_ids:
                skipped += 1
                continue

            for doc_id in docids:
                ret.add(doc_id)

        print(f"Skipped {skipped} queries without relevance judgments")

        return ret

    def __str__(self) -> str:
        return f"top-{self.depth}-run-pool"

def create_subsample(run_dir, ir_datasets_id, depth, output_dir):
    from trectools import TrecRun
    if not (output_dir/"subsample.json").is_file():
        runs = []
        for i in tqdm(glob(f"{run_dir}/*"), "Load Runs"):
            try:
                runs.append(TrecRun(i))
            except:
                pass
        corpus = list(RunPoolCorpusSampler(depth).sample_corpus(ir_datasets_id, runs))
        with open(f"{output_dir}/subsample.json", "w") as f:
            f.write(json.dumps(corpus))
    ret = json.loads((output_dir/"subsample.json").read_text())
    return ret
    



# The code is taken from https://github.com/webis-de/ecir25-corpus-subsampling/blob/main/tests/test_run_pool_corpus_sampler.py
import unittest

import pandas as pd
from trectools import TrecRun

from lsr_benchmark.corpus.corpus_subsampling import RunPoolCorpusSampler
DATASET_ID_FOR_TEST = "disks45/nocr/trec-robust-2004"
RUN_WITH_NO_OVERLAPPING_DOCUMENTS = TrecRun()
RUN_WITH_NO_OVERLAPPING_DOCUMENTS.run_data = pd.DataFrame(
    [
        {"query": "308", "docid": "does-not-exist", "rank": 1, "score": 1},
        {"query": "331", "docid": "does-not-exist", "rank": 1, "score": 1},
    ]
)
RUN_WITH_OVERLAPPING_DOCUMENTS = TrecRun()
RUN_WITH_OVERLAPPING_DOCUMENTS.run_data = pd.DataFrame(
    [
        {"query": "308", "docid": "FBIS4-57944-XXX", "rank": 1, "score": 1},
        {"query": "331", "docid": "FR940413-2-00131-XXX", "rank": 1, "score": 1},
        {"query": "425", "docid": "LA011890-0177-XXX", "rank": 1, "score": 1},
    ]
)

SIZE_POOL_ROBUST_04 = 174787

class TestJudgmentPoolCorpusSampler(unittest.TestCase):
    def test_with_empty_runs(self):
        sampler = RunPoolCorpusSampler(depth=100)

        actual = sampler.sample_corpus(DATASET_ID_FOR_TEST, [])

        # should be the complete judgment pool
        self.assertEqual(SIZE_POOL_ROBUST_04, len(actual))

    def test_with_run_without_overlapping_doc_ids(self):
        expected = set(["does-not-exist"])
        sampler = RunPoolCorpusSampler(depth=100)

        actual = sampler.sample_corpus(DATASET_ID_FOR_TEST, [RUN_WITH_NO_OVERLAPPING_DOCUMENTS])

        for i in expected:
            self.assertIn(i, actual)

        self.assertEqual(SIZE_POOL_ROBUST_04+1, len(actual))

    def test_with_run_with_overlapping_doc_ids(self):
        expected = set(["FBIS4-57944-XXX", "FR940413-2-00131-XXX", "LA011890-0177-XXX"])
        sampler = RunPoolCorpusSampler(depth=100)

        actual = sampler.sample_corpus(DATASET_ID_FOR_TEST, [RUN_WITH_OVERLAPPING_DOCUMENTS])

        for i in expected:
            self.assertIn(i, actual)
        self.assertEqual(SIZE_POOL_ROBUST_04+3, len(actual))

    def test_with_multiple_runs(self):
        expected = set(["FBIS4-57944-XXX", "FR940413-2-00131-XXX", "LA011890-0177-XXX", "does-not-exist"])
        sampler = RunPoolCorpusSampler(depth=100)

        actual = sampler.sample_corpus(
            DATASET_ID_FOR_TEST, [RUN_WITH_OVERLAPPING_DOCUMENTS, RUN_WITH_NO_OVERLAPPING_DOCUMENTS]
        )
        
        for i in expected:
            self.assertIn(i, actual)

        self.assertEqual(SIZE_POOL_ROBUST_04+4, len(actual))

    def test_string_representation_depth_10(self):
        expected = "top-10-run-pool"
        actual = str(RunPoolCorpusSampler(depth=10))

        self.assertEqual(expected, actual)

    def test_string_representation_depth_100(self):
        expected = "top-100-run-pool"
        actual = str(RunPoolCorpusSampler(depth=100))

        self.assertEqual(expected, actual)



__version__ = "0.0.1rc4"
import json
from pathlib import Path
from ir_datasets import registry
from lsr_benchmark.irds import build_dataset, MAPPING_OF_DATASET_IDS, ir_datasets_from_tira
from lsr_benchmark.corpus import materialize_corpus, materialize_queries, materialize_qrels
from click import group, argument
import lsr_benchmark.click

from tirex_tracker import tracking, ExportFormat

SUPPORTED_IR_DATASETS = MAPPING_OF_DATASET_IDS.keys()

from ._commands._evaluate import evaluate
from ._commands._retrieval import retrieval
from ._commands._download import download_embeddings, download_run
from .datasets import TIRA_DATASET_ID_TO_IR_DATASET_ID, IR_DATASET_TO_TIRA_DATASET
import os


def register_to_ir_datasets(dataset=None):
    if dataset and os.path.isdir(dataset):
        if dataset not in registry:
            ds = build_dataset(dataset, False)
            registry.register(dataset, ds)
            registry.register("lsr-benchmark/" + dataset, ds)
    elif dataset and (dataset in ir_datasets_from_tira() or dataset in ir_datasets_from_tira(True)):
        if dataset not in registry:
            ds = build_dataset(dataset, False)

            registry.register(dataset, ds)
            registry.register("lsr-benchmark/" + dataset, ds)
    elif dataset and dataset in IR_DATASET_TO_TIRA_DATASET:
        if ("lsr-benchmark/" + dataset) not in registry:
            ds = build_dataset(IR_DATASET_TO_TIRA_DATASET[dataset], False)
            
            if IR_DATASET_TO_TIRA_DATASET[dataset] not in registry:
                registry.register(IR_DATASET_TO_TIRA_DATASET[dataset], ds)
            registry.register("lsr-benchmark/" + dataset, ds)
    elif dataset and dataset not in SUPPORTED_IR_DATASETS:
        raise ValueError(f"Can not register {dataset}. Supported are: {sorted(IR_DATASET_TO_TIRA_DATASET.keys())}")
    else:
        for k in SUPPORTED_IR_DATASETS:
            irds_id = f"lsr-benchmark/{k}/segmented"
            if irds_id not in registry:
                registry.register(irds_id, build_dataset(k, True))

            irds_id = f"lsr-benchmark/{k}"
            if irds_id not in registry:
                registry.register(irds_id, build_dataset(k, False))


def load(ir_datasets_id: str):
    return build_dataset(ir_datasets_id, False)


@group()
def main():
    pass


def create_subsampled_corpus(directory, config):
    target_directory = directory

    target_directory.mkdir(exist_ok=True)
    with tracking(export_file_path=Path(target_directory) / "dataset-metadata.yml", export_format=ExportFormat.IR_METADATA):
        materialize_corpus(target_directory, config)
        materialize_queries(target_directory, config)
        materialize_qrels(target_directory/"qrels.txt", config)


@main.command()
@argument('directory', type=Path)
def create_lsr_corpus(directory):
    config = json.loads((directory/"config.json").read_text())
    create_subsampled_corpus(directory, config)


@main.command()
def overview():
    overview = json.loads((Path(__file__).parent / "datasets" / "overview.json").read_text())
    df_dataset = []
    overall_size = 0
    overall_datasets = len(overview)
    overall_embeddings = set()
    def f(s):
        return str(int(s/(1024))) + " MB"

    import pandas as pd
    model_to_size = {}

    for dataset_id, stats in overview.items():
        overall_size += int(stats['dataset-size'])
        embeddings_for_dataset = 0
        for embedding, embedding_size in stats['embedding-sizes'].items():
            overall_size += int(embedding_size)
            embeddings_for_dataset += int(embedding_size)
            overall_embeddings.add(embedding)
            model_to_size[embedding] = int(embedding_size) + model_to_size.get(embedding, 0)

        df_dataset += [{"Dataset": TIRA_DATASET_ID_TO_IR_DATASET_ID.get(dataset_id, dataset_id), "Text": f(int(stats['dataset-size'])), "Avg. Embeddings": f(embeddings_for_dataset/len(overall_embeddings))}]
    df_dataset = pd.DataFrame(df_dataset)
    df_dataset.index = ['']*len(df_dataset)
    
    df_embeddings = []
    for k, v in model_to_size.items():
        df_embeddings += [{"Model": k, "Size (avg)": f(v/len(overall_embeddings))}]


    df_embeddings = pd.DataFrame(df_embeddings)
    df_embeddings.index = ['']*len(df_embeddings)

    print(f"Overview of the lsr-benchmark:\n\n\t- {overall_datasets} Datasets with {len(overall_embeddings)} pre-computed embeddings ({f(overall_size)})\n\nDatasets:\n{df_dataset.sort_values('Dataset')}\n\nEmbeddings:\n{df_embeddings}")

main.command()(download_embeddings)
main.command()(download_run)
main.command()(evaluate)
main.command()(retrieval)

if __name__ == '__main__':
    main()
